/*
 * nfsctl	Communicate export information to knfsd.
 *
 * Copyright (C) 1995 Olaf Kirch <okir@monad.swb.de>
 */

#include <stdlib.h>
#include <string.h>
#include "nfslib.h"
#include "exportfs.h"
#include "xio.h"

static int	expsetup(struct nfs_export *ep, struct mexport *exp);
static int	cltsetup(struct nfs_client *cp, struct mclient *clp);

int
export_export(struct mexport *exp)
{
	struct mclient *	clp = exp->m_client;
	struct nfs_export	ee;
	struct nfs_client	cc;

	if (!clp->m_exported) {
		if (!cltsetup(&cc, clp))
			return 0;
		if (nfsaddclient(&cc) < 0)
			return 0;
		clp->m_exported = 1;
	}
	if (!expsetup(&ee, exp))
		return 0;
	if (nfsexport(&ee) < 0)
		return 0;
	exp->m_exported = 1;
	return 1;
}

int
export_unexport(struct mexport *exp)
{
	struct nfs_export	ee;
	int			ok;

	if (!expsetup(&ee, exp) || nfsunexport(&ee) < 0)
		return 0;
	exp->m_exported = 0;
	return ok;
}

static int
cltsetup(struct nfs_client *cp, struct mclient *clp)
{
	int	i;

	if (clp->m_type != MCL_FQDN) {
		xlog(L_ERROR, "internal: can't export non-FQDN host");
		return 0;
	}
	memset(cp, 0, sizeof(*cp));
	strncpy(cp->cl_ident, clp->m_hostname, NFSCLNT_IDMAX);
	cp->cl_naddr = clp->m_naddr;
	for (i = 0; i < cp->cl_naddr && i < NFSCLNT_ADDRMAX; i++)
		cp->cl_addrlist[i] = clp->m_addrlist[i];

	if (clp->m_fhkey.k_type == CLE_KEY_NONE) {
		cp->cl_fhkeytype = CLE_KEY_NULL;
	} else {
		cp->cl_fhkeytype = clp->m_fhkey.k_type;
		cp->cl_fhkeylen = clp->m_fhkey.k_len;
		memcpy(cp->cl_fhkey, clp->m_fhkey.k_key, NFSCLNT_KEYMAX);
	}

	return 1;
}

static int
expsetup(struct nfs_export *ep, struct mexport *exp)
{
	struct mclient		*clp = exp->m_client;

	memset(ep, 0, sizeof(*ep));
	strncpy(ep->ex_path, exp->m_export.e_path, NFS_MAXPATHLEN);
	strncpy(ep->ex_client, clp->m_hostname, NFSCLNT_IDMAX);
	ep->ex_flags = exp->m_export.e_flags;
	ep->ex_anon_uid = exp->m_export.e_anonuid;
	ep->ex_anon_gid = exp->m_export.e_anongid;

	if ((exp->m_export.e_flags & NFSEXP_SIGNFH) &&
	    clp->m_fhkey.k_type == CLE_KEY_NONE) {
		xlog(L_ERROR, "no signature key defined for client %s",
			clp->m_hostname);
		return 0;
	}

	return 1;
}
