/*
 * exportfs.h		Declarations for exportfs and mountd
 *
 * Copyright (C) 1995 Olaf Kirch <okir@monad.swb.de>
 */

#ifndef EXPORTFS_H
#define EXPORTFS_H

#include <nfs/nfsctl.h>
#include <netdb.h>
#include "nfslib.h"

enum {
	MCL_FQDN = 0, MCL_SUBNETWORK, MCL_WILDCARD,
	MCL_NETGROUP, MCL_ANONYMOUS,
	MCL_MAXTYPES
};

struct mclient {
	struct mclient		*m_next;
	char			m_hostname[NFSCLNT_IDMAX+1];
	int			m_type;
	int			m_naddr;
	struct in_addr		m_addrlist[NFSCLNT_ADDRMAX];
	struct in_addr		m_mask;		/* for addr/mask entries */
	struct nfskey		m_fhkey;	/* filehandle MAC key */
	int			m_exported;	/* exported to nfsd */
	int			m_count;
};

struct mexport {
	struct mexport		*m_next;
	struct mclient		*m_client;
	struct exportent	m_export;
	int			m_exported : 1,	/* known to knfsd */
				m_xtabent  : 1,	/* xtab entry exists */
				m_mayexport: 1,	/* derived from xtabbed */
				m_addxtab  : 1;	/* to be added to xtab */
};

extern struct mclient		*clientlist[MCL_MAXTYPES];
extern struct mexport		*exportlist[MCL_MAXTYPES];

struct mclient *client_lookup(char *hname);
struct mclient *client_find(struct hostent *hp);
void		client_add(struct mclient *);
struct mclient *client_dup(struct mclient *clp, struct hostent *hp);
int		client_gettype(char *hname);
int		client_check(struct mclient *clp, struct hostent *hp);
int		client_match(struct mclient *clp, char *hname);
void		client_release(struct mclient *clp);
void		client_freeall(void);

int		export_read(char *fname);
void		export_add(struct mexport *);
struct mexport *export_lookup(char *hname, char *path);
struct mexport *export_find(struct hostent *hp, char *path);
struct mexport *export_create(struct exportent *xep);
void		export_freeall(void);
int		export_export(struct mexport *exp);
int		export_unexport(struct mexport *exp);

int		xtab_read(void);
int		xtab_write(int append);
void		xtab_append(struct mexport *exp);

struct nfskey *	key_lookup(char *hname);

#endif /* EXPORTFS_H */

