/*
 * nfslib.h		General support functions for NFS user-space
 *			programs.
 *
 * Copyright (C) 1995 Olaf Kirch <okir@monad.swb.de>
 */

#ifndef NFSLIB_H
#define NFSLIB_H

#include <sys/types.h>
#include <sys/socket.h>
#include <paths.h>
#include <nfs/nfsconst.h>
#include <nfs/nfsctl.h>
#include <nfs/nfsmac.h>
#include <nfs/nfsfh.h>
#include "xlog.h"

#ifndef _PATH_NFSKEYS
#define _PATH_NFSKEYS		"/etc/nfskeys"
#endif
#ifndef _PATH_EXPORTS
#define _PATH_EXPORTS		"/etc/exports"
#endif
#ifndef _PATH_XTAB
#define _PATH_XTAB		"/var/lib/nfs/xtab"
#endif
#ifndef _PATH_XTABTMP
#define _PATH_XTABTMP		"/var/lib/nfs/xtab.tmp"
#endif
#ifndef _PATH_RMTAB
#define _PATH_RMTAB		"/var/lib/nfs/rmtab"
#endif

enum cle_keytypes {
	CLE_KEY_NONE = -1,
	CLE_KEY_NULL = NFSSIG_NULL,
	CLE_KEY_MD5  = NFSSIG_MD5,
	CLE_KEY_SHA  = NFSSIG_SHA
};

enum cle_maptypes {
	CLE_MAP_IDENT = 0,
	CLE_MAP_FILE,
	CLE_MAP_UGIDD,
};

struct nfskey {
	int		k_type;
	__u8		k_key[64];
	int		k_len;
};

/*
 * Entry in the nfskey database. Currently, keys are used only to sign
 * fh's.
 */
struct nfskeyent {
	char		k_hostname[NFSCLNT_IDMAX+1];
	unsigned char	k_resvd;
	struct nfskey	k_key;
};

/*
 * Data related to a single exports entry as returned by getexportent.
 * FIXME: export options should probably be parsed at a later time to 
 * allow overrides when using exportfs.
 */
struct exportent {
	char		e_hostname[NFSCLNT_IDMAX+1];
	char		e_path[NFS_MAXPATHLEN+1];
	int		e_flags;
	int		e_maptype;
	int		e_anonuid;
	int		e_anongid;
	int *		e_squids;
	int		e_nsquids;
	int *		e_sqgids;
	int		e_nsqgids;
};

struct rmtabent {
	char		r_client[NFSCLNT_IDMAX+1];
	char		r_path[NFS_MAXPATHLEN+1];
};

/*
 * configuration file parsing
 */
void			setexportent(char *fname, char *type);
struct exportent *	getexportent(void);
void			putexportent(struct exportent *xep);
void			endexportent(void);
struct exportent *	mkexportent(char *hname, char *path, char *opts);
void			dupexportent(struct exportent *dst,
					struct exportent *src);
int			updateexportent(struct exportent *eep, char *options);

int			setnfskeyent(char *fname);
struct nfskeyent *	getnfskeyent(void);
void			endnfskeyent(void);
FILE *			fsetnfskeyent(char *fname, char *type);
struct nfskeyent *	fgetnfskeyent(FILE *fp);
void			fputnfskeyent(FILE *fp, struct nfskeyent *kep);
void			fendnfskeyent(FILE *fp);

int			setrmtabent(char *type);
struct rmtabent *	getrmtabent(void);
void			putrmtabent(struct rmtabent *xep);
void			endrmtabent(void);
FILE *			fsetrmtabent(char *fname, char *type);
struct rmtabent *	fgetrmtabent(FILE *fp);
void			fputrmtabent(FILE *fp, struct rmtabent *xep);
void			fendrmtabent(FILE *fp);

/*
 * wildmat borrowed from INN
 */
int			wildmat(char *text, char *pattern);

/*
 * nfsd library functions.
 */
int			nfsctl(int cmd, nfsctl_arg *argp, nfsctl_res *resp);
int			nfssvc(int udp, int tcp);
int			nfsaddclient(nfs_client *clp);
int			nfsdelclient(nfs_client *clp);
int			nfsexport(nfs_export *exp);
int			nfsunexport(nfs_export *exp);
struct knfs_fh *	getfh(int version, char *path, struct sockaddr *addr);
int			nfsdebug_set(unsigned int *debug);
int			nfsdebug_get(unsigned int *debug);
unsigned int		nfsdebug_getflag(const char *name);

#endif /* NFSLIB_H */
