#ifndef SHA_H
#define SHA_H

/* NIST Secure Hash Algorithm */
/* heavily modified from Peter C. Gutmann's implementation */

/* Useful defines & typedefs */

#include <linux/types.h>
#include <asm/byteorder.h>

#define SHA_BLOCKSIZE		64
#define SHA_DIGESTSIZE		20

typedef struct SHA_INFO {
    __u32 digest[5];		/* message digest */
    __u32 count_lo, count_hi;	/* 64-bit bit count */
    __u32 data[16];		/* SHA data buffer */
    int  left;			/* leftover from last update */
} SHA_INFO;

void sha_init(SHA_INFO *);
void sha_update(SHA_INFO *, __u8 *, int);
void sha_final(SHA_INFO *);

#ifndef __KERNEL__
#include <stdio.h>

void sha_stream(SHA_INFO *, FILE *);
void sha_print(SHA_INFO *);
#endif

#endif /* SHA_H */
