/*
 * auth.h		nfsd-specific authentication stuff.
 *
 * Copyright (C) 1995 Olaf Kirch <okir@monad.swb.de>
 */

#ifndef NFSD_AUTH_H
#define NFSD_AUTH_H

/*
 * Various RPC authentication flavors.
 */
#define AUTH_NULL	0
#define AUTH_UNIX	1
#define AUTH_SHORT	2
#define AUTH_DES	3

#ifdef __KERNEL__

typedef struct svc_authops	svc_authops;

typedef struct svc_auth {
	u32		ah_flavor;
	svc_authops *	ah_op;
	u32 *		ah_cbase;	/* credentials */
	int		ah_clen;
	u32 *		ah_vbase;	/* verifier */
	int		ah_vlen;
} svc_auth;

struct svc_buf;		/* forward decl */
struct svc_export;	/* ditto */

/*
 * Standard operations for RPC authentication schemes.
 */
struct svc_authops {
	/* Decode authinfo */
	int		(*decode)(svc_auth *ahp, struct svc_buf *argp);
	void		(*free)(svc_auth *ahp);

	/* Set fsuid/fsgid and supplemental groups */
	int		(*setuser)(svc_auth *ahp, struct svc_export *exp);

	/* Check client credential or verifier */
	int		(*verify)(svc_auth *ahp, struct svc_buf *argp,
					u32 *errp);

	/* Add verifier to response. */
	int		(*addverf)(svc_auth *ahp, struct svc_buf *resp,
					u32 *errp);
};

/*
 * Decoded AUTH_UNIX data. This is different from what's in the RPC lib.
 */
#define NGRPS		16
struct authunix_parms {
	u32		aup_stamp;
	u32		aup_uid;
	u32		aup_gid;
	u32		aup_len;
	u32		aup_gids[NGRPS];
};

svc_authops *		auth_getops(u32 flavor);


#endif /* __KERNEL__ */

#endif /* NFSD_AUTH_H */
