/*
 * nfsctl.h		This file holds all declarations for the knfsd
 *			syscall interface.
 *
 * Copyright (C) 1995 Olaf Kirch <okir@monad.swb.de>
 */

#include <linux/types.h>
#include <linux/socket.h>
/* #include <linux/unistd.h> */
/* #include <linux/fs.h> */
#include <nfs/nfsconst.h>
#include <nfs/export.h>
#include <nfs/nfsfh.h>
#include <nfs/auth.h>

#ifndef __NFSCTL_H__
#define __NFSCTL_H__

/*
 * Version of the syscall interface
 */
#define NFSCTL_VERSION		0x0102

/*
 * These are the commands understood by nfsctl().
 */
#define NFSCTL_SVC		0	/* This is a server process. */
#define NFSCTL_ADDCLIENT	1	/* Add an NFS client. */
#define NFSCTL_DELCLIENT	2	/* Remove an NFS client. */
#define NFSCTL_EXPORT		3	/* export a file system. */
#define NFSCTL_UNEXPORT		4	/* unexport a file system. */
#define NFSCTL_UGIDUPDATE	5	/* update a client's uid/gid map. */
#define NFSCTL_GETFH		6	/* get an fh (used by mountd) */
#define NFSCTL_FHOPEN		7	/* open a file by fh (used by lockd) */
#define NFSCTL_AUTHD		8	/* used by the authentication daemon */
#define NFSCTL_DEBUG		9	/* debugging flags */


/* SVC */
#define NFS_MAXSVCFDS		4
typedef struct nfs_svc {
	int			svc_fd[NFS_MAXSVCFDS];
	int			svc_nrfds;
} nfs_svc;
#define svc_udp		svc_fd[0]
#define svc_tcp		svc_fd[1]

/* ADDCLIENT/DELCLIENT */
typedef struct nfs_client {
	char			cl_ident[NFSCLNT_IDMAX+1];
	int			cl_naddr;
	struct in_addr		cl_addrlist[NFSCLNT_ADDRMAX];
	int			cl_fhkeytype;
	int			cl_fhkeylen;
	unsigned char		cl_fhkey[NFSCLNT_KEYMAX];
} nfs_client;

/* EXPORT/UNEXPORT */
typedef struct nfs_export {
	char			ex_path[NFS_MAXPATHLEN+1];
	char			ex_client[NFSCLNT_IDMAX+1];
	int			ex_flags;
	uid_t			ex_anon_uid;
	gid_t			ex_anon_gid;
} nfs_export;

/* UGIDUPDATE */
typedef struct nfs_ugidmap {
	char *			ug_ident;
	uid_t			ug_uidbase;
	int			ug_uidlen;
	uid_t *			ug_udimap;
	uid_t			ug_gidbase;
	int			ug_gidlen;
	gid_t *			ug_gdimap;
} nfs_uidmap;

/* GETFH */
typedef struct nfsctl_getfh_arg {
	struct sockaddr		gf_addr;
	char			gf_path[NFS_MAXPATHLEN+1];
	int			gf_version;
} nfsctl_getfh_arg;

/* OPENFH */
typedef struct nfsctl_openarg {
	struct knfs_fh		op_fh;
	struct sockaddr		op_addr;
} nfsctl_openarg;

/*
 * This is the argument union.
 */
typedef struct nfsctl_arg {
	int			ca_version;	/* safeguard */
	union {
		nfs_svc			u_svc;
		nfs_client		u_client;
		nfs_export		u_export;
		/* nfsctl_ugidmap	u_umap; */
		nfsctl_getfh_arg	u_getfh;
		struct auth_args	u_authd;
		unsigned int		u_debug;
	} u;
#define ca_svc		u.u_svc
#define ca_client	u.u_client
#define ca_export	u.u_export
#define ca_umap		u.u_umap
#define ca_getfh	u.u_getfh
#define ca_authd	u.u_authd
#define ca_debug	u.u_debug
} nfsctl_arg;

typedef union nfsctl_res {
	struct knfs_fh		cr_getfh;
	struct auth_args	cr_authd;
	unsigned int		cr_debug;
} nfsctl_res;

#ifdef __KERNEL__
/*
 * Kernel syscall implementation.
 */
int		exp_addclient(nfs_client *ncp);
int		exp_delclient(nfs_client *ncp);
int		exp_export(nfs_export *nxp);
int		exp_unexport(nfs_export *nxp);

#endif /* __KERNEL__ */

#endif /* __NFSCTL_H__ */
