/*
 * nfsfh.h		This file describes the layout of the file handles
 *			as passed over the wire.
 *
 * Copyright (C) 1995 Olaf Kirch <okir@monad.swb.de>
 */

#ifndef __NFSFH_H__
#define __NFSFH_H__

#include <linux/types.h>
#include <linux/fs.h>
#include <nfs/nfsconst.h>

/*
 * This is our NFSv2 file handle. This currently assumes that dev_t is
 * 16 bit, and ino_t is 32 bit. This leaves 128 bit for the cookie, 
 * just enough for an MD5 hash. If dev_t should be increased to 32bit
 * one fine day, or a MAC algorithm with a larger digest size (such as
 * SHA) is used, only part of the digest is included in the handle.
 * You can't call this safe, but that's the way things go. We're all
 * looking forward to NFSv3.
 *
 * The xdev and xino fields are currently used to transport the dev/ino
 * of the export point. This idea may not be as good as it sounds. I'll
 * have to do some more thinking about this.
 */
struct nfs_fhbase {
	dev_t			fb_dev;
	dev_t			fb_xdev;
	ino_t			fb_ino;
	ino_t			fb_xino;
	__u32			fb_version;
};

#define NFS2_FHCOOKIE_SIZE	(NFS2_FHSIZE - sizeof(struct nfs_fhbase))
struct nfs2_fh {
	struct nfs_fhbase	fh_base;
	__u8			fh_cookie[NFS2_FHCOOKIE_SIZE];
};

#define NFS3_FHCOOKIE_SIZE	(NFS3_FHSIZE - sizeof(struct nfs_fhbase))
struct nfs3_fh {
	struct nfs_fhbase	fh_base;
	__u8			fh_cookie[NFS3_FHCOOKIE_SIZE];
};


#define NFS_MAX_FHCOOKIE_SIZE	NFS3_FHCOOKIE_SIZE
struct knfs_fh {
	struct nfs_fhbase	fh_base;
	__u8			fh_cookie[NFS_MAX_FHCOOKIE_SIZE];
};

#define fh_dev			fh_base.fb_dev
#define fh_xdev			fh_base.fb_xdev
#define fh_ino			fh_base.fb_ino
#define fh_xino			fh_base.fb_xino
#define fh_version		fh_base.fb_version

#endif /* __NFSFH_H__ */
