/*
 * nfsstat.c	procfs-based user access to knfsd statistics
 *
 * /proc/net/nfssrv
 * Format:
 *	net <packets> <udp> <tcp> <tcpconn>
 *	rpc <packets> <badfmt> <badclnt>
 *	auth <flavor> <creds> <upcalls> <badauth> <badverf> <authrej>
 *	fh  <hits> <misses> <avg_util> <stale> <cksum> <badcksum>
 *	rc <hits> <misses> <nocache>
 *	proto <version> <nrprocs>
 *	<calls> <time_msec>
 *	... (for each procedure and protocol version)
 *
 * Copyright (C) 1995, 1996 Olaf Kirch <okir@monad.swb.de>
 */

#ifdef MODULE
#include <linux/modversions.h>
#include <linux/module.h>
#include <linux/version.h>
#endif

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/proc_fs.h>
#include <linux/stat.h>

#include "nfsd.h"

struct nfsd_stats	nfsdstats;

static int		nfsd_get_info(char *, char **, off_t, int, int);

#ifndef PROC_NET_NFSSRV
# define PROC_NET_NFSSRV	0
#endif

static struct proc_dir_entry proc_nfssrv = {
	PROC_NET_NFSSRV, 6, "nfssrv",
	S_IFREG | S_IRUGO, 1, 0, 0,
	6, &proc_net_inode_operations,
	nfsd_get_info
};

static int
nfsd_get_info(char *buffer, char **start, off_t offset, int length, int dummy)
{
	struct svc_version	*vp;
	struct svc_proc		*pp;
	int			len, i, j;

	len = sprintf(buffer,
		"net %d %d %d %d\n",
			nfsdstats.netcnt,
			nfsdstats.netudpcnt,
			nfsdstats.nettcpcnt,
			nfsdstats.nettcpconn);
	len += sprintf(buffer + len,
		"rpc %d %d %d\n",
			nfsdstats.rpccnt,
			nfsdstats.rpcbadfmt,
			nfsdstats.rpcbadauth,
			nfsdstats.rpcbadclnt);
	len += sprintf(buffer + len,
		"fh %d %d %d %d %d\n",
			nfsdstats.fhhits,
			nfsdstats.fhmisses,
			nfsdstats.fhstale,
			nfsdstats.fhcksum,
			nfsdstats.fhbadcksum);
	len += sprintf(buffer + len,
		"rc %d %d %d\n",
			nfsdstats.rchits,
			nfsdstats.rcmisses,
			nfsdstats.rcnocache);

	for (i = 0, vp = nfsd_version; i <= NFSD_MAXVERS; i++, vp++) {
		if ((pp = vp->sv_procs) == NULL)
			continue;
		len += sprintf(buffer + len, "nfsv%d", i);
		for (j = 0; j < vp->sv_nproc; j++, pp++) {
			len += sprintf(buffer + len, " %d", pp->p_count);
		}
		buffer[len++] = '\n';
	}

	if (offset >= len) {
		*start = buffer;
		return 0;
	}
	*start = buffer + offset;
	len -= offset;
	if (len > length)
		len = length;
	return len;
}

#if PROC_NET_NFSSRV != 0
void
nfsstat_init(void)
{
	proc_net_register(&proc_nfssrv);
}

void
nfsstat_shutdown(void)
{
	proc_net_unregister(PROC_NET_NFSSRV);
}
#else
void
nfsstat_init(void)
{
	proc_register_dynamic(&proc_net, &proc_nfssrv);
}

void
nfsstat_shutdown(void)
{
	if (proc_nfssrv.low_ino != 0)
		proc_unregister(&proc_net, proc_nfssrv.low_ino);
}
#endif
