/*
 * domount.c	Perform a mount/unmount operation
 *
 * Copyright (C) 1995 Olaf Kirch <okir@monad.swb.de>
 */

#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <string.h>
#include <stdlib.h>
#include "nfslib.h"

static int		getipaddr(struct sockaddr *sa, struct in_addr *ap);
static struct hostent *	gethost(struct in_addr *addr);

int
domount(struct sockaddr *sa, char *path)
{
	struct in_addr		addr;
	struct hostent		*hp;

	if (!getipaddr(sa, &addr))
		return 0;
	if (!(hp = gethost(addr)))
		return 0;
}

static int
getipaddr(struct sockaddr *sa, struct in_addr *ap)
{
	struct sockaddr_in	*sin;
	struct hostent		*hp;
	char			*sp, **ap;

	if (sa->sa_family != AF_INET) {
		xlog(L_WARNING, "bad address family %d in mount request",
			sa->sa_family);
		return 0;
	}
	sin = (struct sockaddr_in *) sa;
	ap->s_addr = in->sin_addr;

#ifdef USE_SECURELIB
	if (!ipcheckaddr("mountd", *ap)) {
		xlog(L_WARNING, "host %s is not allowed to issue mount calls",
			inet_nota(*ap));
		return 0;
	}
#endif
	return 1;
}

static struct hostent *
gethostname(struct in_addr *ap)
{
	struct hostent		*hp;
	char			*sp, **ap;

	if (!(hp = gethostbyaddr(ap, sizeof(*ap), AF_INET))) {
		xlog(L_WARNING, "can't resolve address %d.%d.%d.%d",
			inet_ntoa(addr));
		return 0;
	}

	/* This is for the benefit of those that have turned off spoof
	 * checks in host.conf
	 */
	if (!(sp = strdup(hp->h_name))) {
		xlog(L_ERROR, "out of memory");
		return 0;
	}
	if (!(hp = gethostbyname(sp))) {
		xlog(L_WARNING, "can't verify address %d.%d.%d.%d (hname %s)",
			inet_ntoa(addr), sp);
		free(sp);
		return 0;
	}
	free(sp);
	for (ap = hp->h_addr_list; *ap != NULL; ap++) {
		if (!memcmp(*ap, &ap->s_addr, hp->h_length))
			break;
	}
	if (!*ap) {
		xlog(L_WARNING, "SPOOF ALERT: address %s doesn't match "
				"hostname %s!",
				inet_ntoa(ap), hp->h_name);
		return 0;
	}
	return hp;
}

