/*
 * mountlist	Manage /etc/rmtab for mountd.
 *
 * Copyright (C) 1995 Olaf Kirch <okir@monad.swb.de>
 */

#include <sys/stat.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include "xmalloc.h"
#include "exportfs.h"
#include "mountd.h"

#define _PATH_RMTABTMP	_PATH_RMTAB ".tmp"

void
mountlist_add(struct mexport *exp)
{
	struct rmtabent	xe;

	strcpy(xe.r_client, exp->m_client->m_hostname);
	strcpy(xe.r_path, exp->m_export.e_path);
	if (!setrmtabent("a"))
		return;
	putrmtabent(&xe);
	endrmtabent();
}

void
mountlist_del(struct mexport *exp)
{
	struct rmtabent	*rep;
	FILE		*fp;
	char		*hname = exp->m_client->m_hostname,
			*path = exp->m_export.e_path;

	if (!setrmtabent("r"))
		return;
	if (!(fp = fsetrmtabent(_PATH_RMTABTMP, "w"))) {
		endrmtabent();
		return;
	}
	while ((rep = getrmtabent()) != NULL) {
		if (strcmp(rep->r_client, hname) || strcmp(rep->r_path, path))
			fputrmtabent(fp, rep);
	}
	if (rename(_PATH_RMTABTMP, _PATH_RMTAB) < 0) {
		xlog(L_ERROR, "couldn't rename %s to %s",
				_PATH_RMTABTMP, _PATH_RMTAB);
	}
	endrmtabent();	/* close & unlink */
	fendrmtabent(fp);
}

void
mountlist_del_all(struct sockaddr_in *sin)
{
	struct in_addr	addr = sin->sin_addr;
	struct hostent	*hp;
	struct rmtabent	*rep;
	struct mexport	*exp;
	FILE		*fp;

	if (!(hp = gethostbyaddr((char *)&addr, sizeof(addr), AF_INET))) {
		xlog(L_ERROR, "can't get hostname of %s", inet_ntoa(addr));
		return;
	}
	if (!setrmtabent("r"))
		return;
	if (!(fp = fsetrmtabent(_PATH_RMTABTMP, "w"))) {
		endrmtabent();
		return;
	}
	while ((rep = getrmtabent()) != NULL) {
		if (!strcmp(rep->r_client, hp->h_name) &&
		    (exp = auth_authenticate("umountall", sin, rep->r_path)))
			continue;
		fputrmtabent(fp, rep);
	}
	if (rename(_PATH_RMTABTMP, _PATH_RMTAB) < 0) {
		xlog(L_ERROR, "couldn't rename %s to %s",
				_PATH_RMTABTMP, _PATH_RMTAB);
	}
	endrmtabent();	/* close & unlink */
	fendrmtabent(fp);
}

mountlist
mountlist_list(void)
{
	static mountlist	mlist = NULL;
	static time_t		last_mtime = 0;
	mountlist		m;
	struct rmtabent		*rep;
	struct stat		stb;

	if (stat(_PATH_RMTAB, &stb) < 0) {
		xlog(L_ERROR, "can't stat %s", _PATH_RMTAB);
		return NULL;
	}
	if (stb.st_mtime != last_mtime) {
		while (mlist) {
			mlist = (m = mlist)->ml_next;
			xfree(m->ml_hostname);
			xfree(m->ml_directory);
			xfree(m);
		}
		last_mtime = stb.st_mtime;
	}

	setrmtabent("r");
	while ((rep = getrmtabent()) != NULL) {
		m = (mountlist) xmalloc(sizeof(*m));
		m->ml_hostname = xstrdup(rep->r_client);
		m->ml_directory = xstrdup(rep->r_path);
		m->ml_next = mlist;
		mlist = m;
	}
	endrmtabent();

	return mlist;
}
