/*
 * getfh		get the FH for a given client and path. This
 *			function takes the NFS protocol version number
 *			as an additional argument.
 *
 *			This function has nothing in common with the
 *			SunOS getfh function, which is a front-end to
 *			the RPC mount call.
 *
 * Copyright (C) 1995 Olaf Kirch <okir@monad.swb.de>
 */

#include <errno.h>
#include <nfs/nfsfh.h>
#include <nfs/nfsctl.h>
#include "nfslib.h"

struct knfs_fh *
getfh(int version, char *path, struct sockaddr *addr)
{
	static nfsctl_res	res;
	nfsctl_arg		arg;
	nfsctl_getfh_arg	*fa = &arg.ca_getfh;

	if (strlen(path) > NFS_MAXPATHLEN) {
		errno = ENAMETOOLONG;
		return NULL;
	}
	arg.ca_version = NFSCTL_VERSION;
	strcpy(fa->gf_path, path);
	memcpy(&fa->gf_addr, addr, sizeof(fa->gf_addr));
	fa->gf_version = version;
	if (nfsctl(NFSCTL_GETFH, &arg, &res) < 0)
		return NULL;
	return &res.cr_getfh;
}
