/*
 * nfsdebug.c		Turn knfsd debugging on or off
 *
 * Copyright (C) 1995, 1996 Olaf Kirch <okir@monad.swb.de>
 */

#include <nfs/nfsctl.h>
#include <nfs/nfsd.h>
#include "nfslib.h"

static struct dflags {
	char		*name;
	unsigned int	value;
} dflags[] = {
    {	"sock",		NFSDBG_SOCK	},
    {	"fh",		NFSDBG_FH	},
    {	"export",	NFSDBG_EXPORT	},
    {	"svc",		NFSDBG_SVC	},
    {	"proc",		NFSDBG_PROC	},
    {	"fileop",	NFSDBG_FILEOP	},
    {	"auth",		NFSDBG_AUTH	},
    {	"repcache",	NFSDBG_REPCACHE	},
    {	NULL,		0		}
};

int
nfsdebug_set(unsigned int *debug)
{
	nfsctl_arg	arg;
	nfsctl_res	res;
	int		ret;

	arg.ca_version = NFSCTL_VERSION;
	arg.ca_debug = *debug;
	ret = nfsctl(NFSCTL_DEBUG, &arg, &res);
	if (ret >= 0)
		*debug = res.cr_debug;
	return ret;
}

int
nfsdebug_get(unsigned int *debug)
{
	*debug = NFSDBG_NOCHANGE;
	return nfsdebug_set(debug);
}

unsigned int
nfsdebug_getflag(const char *name)
{
	struct dflags	*dp;

	for (dp = dflags; dp->name; dp++)
		if (!strcmp(dp->name, name))
			return dp->value;
	return 0;
}
