/*
 * Get or set knfsd debug flags
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#include <stdio.h>
#include <getopt.h>
#include "nfslib.h"

static void
usage(int excode)
{
	fprintf(stderr, "usage: nfsdebug [-cds] flags ...\n");
	exit (excode);
}

int
main(int argc, char **argv)
{
	int		opt_s = 0,
			opt_d = 0,
			opt_c = 0;
	unsigned int	flags = 0, oflags;
	int		c, ret;

	while ((c = getopt(argc, argv, "cdhs")) != EOF) {
		switch (c) {
		case 'c':
			opt_c = 1;
			break;
		case 'd':
			opt_d = 1;
			break;
		case 'h':
			usage(0);
		case 's':
			opt_s = 1;
			break;
		default:
			fprintf(stderr, "nfsdebug: unknown option -%c\n",
						optopt);
			usage(1);
		}
	}

	if (opt_c + opt_d + opt_s > 1) {
		fprintf(stderr, "You can use at most one of -c, -d, -s\n");
		usage(1);
	}

	for (; optind < argc; optind++) {
		unsigned int	temp;

		if (!(temp = nfsdebug_getflag(argv[optind]))) {
			fprintf(stderr, "nfsdebug: unknown flag %s\n",
						argv[optind]);
		}
		flags |= temp;
	}

	if (opt_c && flags) {
		fprintf(stderr, "nfsdebug: -c clears all flags; your can't\n"
				"specify any flags with this option.\n");
		usage(1);
	}

	if (opt_c || opt_s) {
		ret = nfsdebug_set(&flags);
	} else {
		if (nfsdebug_get(&oflags) < 0) {
			perror("can't get knfsd debug flags");
			return 1;
		}
		if (opt_d)
			oflags &= ~flags;
		else
			oflags |= flags;
		ret = nfsdebug_set(&oflags);
	}

	if (ret < 0) {
		perror("can't set knfsd debug flags");
		return 1;
	}

	return 0;
}
